<?php
// $Id: ajax_validation.pages.inc,v 1.1.2.2 2008/09/11 22:17:44 tapocol Exp $

/**
 * @file
 */

/**
 * This is a page callback that is called for running AJAX submissions
 */
function ajax_validation_page() {
  // Inside this if statement is revised code from drupal_get_form and drupal_process_form
  if (!empty($_POST['form_id']) && !empty($_POST['form_build_id'])) {
    $form_id = $_POST['form_id'];

    $form_state = array('storage' => NULL, 'submitted' => FALSE);

    // Prepare arguments for drupal_retrieve call later on
    $menu_item = menu_get_item($_GET['action']);
    $args = $menu_item['page_arguments'];
    foreach ($args as $key => $arg) {
      $args[$key] = (array) $arg;
    }
    array_unshift($args, NULL);

    $cacheable = FALSE;

    // If the incoming $_POST contains a form_build_id, we'll check the
    // cache for a copy of the form in question. If it's there, we don't
    // have to rebuild the form to proceed. In addition, if there is stored
    // form_state data from a previous step, we'll retrieve it so it can
    // be passed on to the form processing code.
    if (isset($_POST['form_id']) && $_POST['form_id'] == $form_id && !empty($_POST['form_build_id'])) {
      $form = form_get_cache($_POST['form_build_id'], $form_state);
    }

    // If the previous bit of code didn't result in a populated $form
    // object, we're hitting the form for the first time and we need
    // to build it from scratch.
    if (!isset($form)) {
      $form_state['post'] = $_POST;
      // Use a copy of the function's arguments for manipulation
      $args_temp = $args;
      $args_temp[0] = &$form_state;
      array_unshift($args_temp, $form_id);
      $form = call_user_func_array('drupal_retrieve_form', $args_temp);
      $form_build_id = 'form-'. md5(mt_rand());
      $form['#build_id'] = $form_build_id;
      drupal_prepare_form($form_id, $form, $form_state);
      // Store a copy of the unprocessed form for caching and indicate that it
      // is cacheable if #cache will be set.
      $original_form = $form;
      $cacheable = TRUE;
      unset($form_state['post']);
    }
    $form['#post'] = $_POST;
    $form_state['values'] = array();

    $form = form_builder($form_id, $form, $form_state);
    // Only process the form if it is programmed or the form_id coming
    // from the POST data is set and matches the current form_id.
    if ((!empty($form['#programmed'])) || (!empty($form['#post']) && (isset($form['#post']['form_id']) && ($form['#post']['form_id'] == $form_id)))) {
      drupal_validate_form($form['#post']['form_id'], $form, $form_state);
    }

    $obj->success = is_null(form_get_errors());
    if (!$obj->success) {
      $obj->errors->elements = array_keys(form_get_errors());
      $obj->errors->button = $form_state['buttons']['button'][0]['#id'];
      $obj->errors->messages = array_values(form_get_errors());
      $obj->errors->button_message = format_plural(count($obj->errors->messages), 'There is %count error.', 'There are %count errors.', array('%count' => count($obj->errors->messages)));
      $obj->errors->button_message .= ' '. (variable_get('ajax_validation_error_placement', 'field') == 'field' ? t('Please review the highlighted field(s).') : t('Please review the error box above the form.'));
    }
  }
  // If it is not a valid submission, then return an error.
  else {
    $obj->success = FALSE;
    $obj->errors->elements = array('form');
    $obj->errors->messages = array('There is a problem with validating this form. You may need to reload the page.');
  }

  drupal_json($obj);
  unset($_SESSION['messages']); // Clears messages, so errors do not show up on the next page

  exit;
}